use std::collections::VecDeque;
use std::io::Write;

use cpal::{FromSample, Sample, SampleRate};

use ringbuf::{traits::*, LocalRb};

use rustfft::{num_complex::Complex, FftPlanner};

use super::ForeignTwoStateCarrier;

pub fn take_input_pcm<T>(
    pcm: &[T],
    sample_rate: SampleRate,
    channel: &tokio::sync::mpsc::Sender<Vec<f64>>,
    pcm_buffer: &mut VecDeque<Complex<f64>>,
    carrier: &mut ForeignTwoStateCarrier,
) where
    T: ToString + Sample + FromSample<f64> + std::fmt::Debug,
    f64: FromSample<T>,
{
    let fft_size = (sample_rate / carrier.symbol_rate).round() as u32;

    pcm_buffer.extend(pcm.iter().map(|s| Complex {
        re: s.to_sample::<f64>(),
        im: 0.0f64,
    }));

    let mut fft_planner = FftPlanner::<f64>::new();
    let fft_machine = fft_planner.plan_fft_forward(fft_size); // TODO: cfgable
    let mut fft_frame = pcm_buffer[..fft_size];

    // Zero-pad for a better freq resolution. 8192 fft should be fast enough
    if fft_size < 8192 {
        fft_frame.extend([0.0_f64].repeat(8192 - fft_size));
    }

    let mut window_offset = 0;

    while !carrier.acquired
        && pcm_buffer.len() >= fft_size
        && window_offset < fft_size
    {
        fft_machine.process(&mut fft_frame);

        let frame_index = fft_size * carrier.clock_hz / sample_rate.0 as usize;
        if let Some(bin) = fft_frame.get(frame_index) {
            // yay
        }
    }

    while carrier.acquired && pcm_buffer.len() >= fft_size {
        fft_machine.process(&mut fft_frame);

        /*for (i, s) in fft_frame.iter().enumerate() {
            let freq = sample_rate.0 as usize * i / fft_size;
            let ampl = s.norm();
            let confidence = ampl;
            if ampl > 50.0 {
                println!("{:?}: {:?}", freq, ampl);
            }

            // soon: confidence = ampl / |freq_target - freq_observed|
        }*/
    }
}

fn ampl_between_bins_quadratic(
    fft_res: VecDeque<Complex<f64>>,
    freq: f64,
) -> f64 {
    todo!();
}
